package com.starbuds.server.api.agent.extension;

import cn.hutool.core.codec.Base64;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.agent.common.pojo.admin.TokenParam;
import com.starbuds.server.api.agent.constant.HttpError;
import com.starbuds.server.api.agent.inf.PermitGuest;
import com.starbuds.server.api.agent.service.admin.dao.AgentAdminTokenDAO;
import com.starbuds.server.api.agent.service.admin.provider.TokenProvider;
import com.starbuds.server.api.agent.util.SpringContextUtil;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.SignType;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jboss.resteasy.core.ResourceMethodInvoker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.security.DenyAll;
import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.core.Context;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Map;

/**
 * 接口访问安全拦截器
 *
 * @author Lay
 * @date 2019-02-18
 */
@Slf4j
@Component
@Provider
public class SecurityInterceptor implements ContainerRequestFilter {

    @Context
    HttpServletRequest request;

    @Autowired
    AgentAdminTokenDAO agentAdminTokenDAO;

    static final long SIGN_TIMEOUT = 60 * 1000L;

    @Override
    public void filter(ContainerRequestContext requestContext) throws IOException {
        String reqPath = requestContext.getUriInfo().getPath();

        // 打印请求日志
        String qs = paramsMapToQueryString(request.getParameterMap());
        String headers = getHeaderString();
        log.info(String.format("\n请求接口:%s\n请求方式:%s\n请求参数:%s\n请求头:%s", reqPath, requestContext.getMethod(), qs, headers));

        // 权限校验部分
        ResourceMethodInvoker methodInvoker = (ResourceMethodInvoker) requestContext.getProperty("org.jboss.resteasy.core.ResourceMethodInvoker");
        Method method = methodInvoker.getMethod();

        // 如果不是除了所有的权限
        if (!method.isAnnotationPresent(PermitAll.class)) {
            // 如果是拒绝所有访问
            if (method.isAnnotationPresent(DenyAll.class)) {
                requestContext.abortWith(HttpError.ACCESS_FORBIDDEN);
                return;
            }

            // 请求头解包
            HeaderPackage headerPackage = getHeaderPackage();
            if (headerPackage == null) {
                log.error("未获取到解码后的请求头信息");
                requestContext.abortWith(HttpError.ACCESS_DENIED);
                return;
            }

            // 请求头时间校验
            if (!SpringContextUtil.isDev() && headerPackage.getTimestamp() + SIGN_TIMEOUT < System.currentTimeMillis()) {
                log.error("签名已过期");
                requestContext.abortWith(HttpError.ACCESS_DENIED);
                return;
            }

            if (AppType.Admin != headerPackage.getAppType()) {
                log.error("AppType 错误");
                requestContext.abortWith(HttpError.ACCESS_DENIED);
                return;
            }

            // 请求头对象注入
            requestContext.setProperty("header", headerPackage);

            // 解析令牌
            TokenParam tokenParam = TokenProvider.decryptToken(headerPackage.getToken());
            if (tokenParam != null && tokenParam.getAdminId() > 0) {

                if (!method.isAnnotationPresent(PermitGuest.class) && false) {
                    requestContext.abortWith(HttpError.ACCESS_DENIED);
                    return;
                }

                // token失效
                requestContext.setProperty("token", tokenParam);
                if (!tokenParam.getToken().equals(agentAdminTokenDAO.getToken(tokenParam.getAdminId()))) {
                    requestContext.abortWith(HttpError.TOKEN_INVALID);
                    return;
                }

                // 令牌注入上下文
                requestContext.setProperty("token", tokenParam);
            } else {
                if (!method.isAnnotationPresent(PermitGuest.class)) {
                    requestContext.abortWith(HttpError.TOKEN_INVALID);
                    return;
                }
            }
        }
    }

    private HeaderPackage getHeaderPackage() {
        String pkg = request.getHeader("pkg");

        if (StringUtils.isNotBlank(pkg)) {
            // 尝试解包
            try {
                pkg = Base64.decodeStr(pkg);
                log.info("decode pkg: {}", pkg);
                HeaderPackage headerPackage = JSON.parseObject(pkg, HeaderPackage.class);

                // 参数校验
                if (headerPackage.getClientType() == null ||
                        headerPackage.getAppType() == null ||
                        headerPackage.getPackageName() == null ||
                        headerPackage.getTimestamp() == null ||
                        headerPackage.getSignType() == null ||
                        headerPackage.getSign() == null ||
                        headerPackage.getToken() == null ||
                        headerPackage.getDeviceId() == null) {
                    log.error("请求头参数不完整 {} {} {} {} {} {} {} {}", headerPackage.getClientType(), headerPackage.getAppType(), headerPackage.getPackageName(), headerPackage.getTimestamp(), headerPackage.getSignType(), headerPackage.getSign(), headerPackage.getToken(), headerPackage.getDeviceId());
                    return null;
                }

                // 签名校验
                if (verifySign(headerPackage)) {
                    log.info("headerPackage:{}", JSON.toJSONString(headerPackage));
                    return headerPackage;
                }
                log.error("签名校验不通过");
            } catch (Exception e) {
                log.error("请求头解包失败", e);
            }
        }

        return null;
    }

    /**
     * 签名校验
     *
     * @return
     */
    private boolean verifySign(HeaderPackage headerPackage) {
        String toSign = headerPackage.toQueryString();

        String sign = "";
        switch (headerPackage.getSignType()) {
            case SignType.MD5:
                sign = SecureUtil.md5(toSign + "pq" + SignType.MD5);
                break;
            case SignType.SHA1:
                sign = SecureUtil.sha1(toSign + "pq" + SignType.SHA1);
                break;
            case SignType.SHA256:
                sign = SecureUtil.sha256(toSign + "pq" + SignType.SHA256);
                break;
            default:
                break;
        }

        return StringUtils.isNotBlank(sign) && sign.equals(headerPackage.getSign());
    }

    private String paramsMapToQueryString(Map<String, String[]> map) {
        if (map == null) {
            return "";
        }

        StringBuilder sbQuery = new StringBuilder();
        for (String key : map.keySet()) {
            for (String val : map.get(key)) {
                sbQuery.append(String.format("&%s=%s", key, val));
            }
        }
        if (sbQuery.length() > 0) {
            return sbQuery.toString().substring(1);
        }

        return "";
    }

    private String getHeaderString() {
        Enumeration HeaderKey = request.getHeaderNames();
        StringBuilder sbHeader = new StringBuilder();
        while (null != HeaderKey && HeaderKey.hasMoreElements()) {
            String headerKey = (String) HeaderKey.nextElement();
            sbHeader.append(String.format("%s=%s&", headerKey, request.getHeader(headerKey)));
        }
        return sbHeader.toString();
    }
}
