package com.starbuds.server.api.agent.service.admin.impl;

import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.api.agent.common.pojo.admin.TokenParam;
import com.starbuds.server.api.agent.common.pojo.db.AgentAdminSecurityDO;
import com.starbuds.server.api.agent.common.pojo.db.AgentAdminTokenDO;
import com.starbuds.server.api.agent.common.service.admin.AgAdminSecurityService;
import com.starbuds.server.api.agent.service.admin.dao.AgentAdminSecurityDAO;
import com.starbuds.server.api.agent.service.admin.dao.AgentAdminTokenDAO;
import com.starbuds.server.api.agent.service.admin.provider.TokenProvider;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 14:09
 * @Description 安全业务实现
 */
@Slf4j
@Service
public class AgAdminSecurityServiceImpl implements AgAdminSecurityService {

    @Autowired
    AgentAdminSecurityDAO adminSecurityDAO;

    @Autowired
    AgentAdminTokenDAO agentAdminTokenDAO;

    /**
     * 根据账号查询
     *
     * @param account
     * @return
     */
    @Override
    public AgentAdminSecurityDO getSecurityByAccount(String account) {
        return adminSecurityDAO.selectByAccount(account);
    }

    /**
     * 根据账号id查询
     *
     * @param adminId
     * @return
     */
    @Override
    public AgentAdminSecurityDO getSecurityByAdminId(Long adminId) {
        return adminSecurityDAO.selectById(adminId);
    }

    /**
     * 密码检查
     *
     * @param password
     * @param dbPass
     * @return
     */
    @Override
    public boolean checkPass(String password, String dbPass) {
        return dbPass.equals(encryptPassword(password));
    }

    /**
     * 更新登录令牌
     *
     * @param adminId
     * @param clientType
     * @param token
     * @return
     */
    @Override
    public boolean setToken(Long adminId, Integer clientType, String token) {
        try {
            AgentAdminTokenDO adminTokenDO = new AgentAdminTokenDO();
            adminTokenDO.setAdminId(adminId);
            adminTokenDO.setClientType(clientType);
            adminTokenDO.setTokenValue(token);
            adminTokenDO.setTokenCreateTime(System.currentTimeMillis());
            agentAdminTokenDAO.insertOrUpdate(adminTokenDO);
            return true;
        } catch (Exception e) {
            log.error("登录令牌更新失败", e);
        }
        return false;
    }

    /**
     * 加密令牌
     *
     * @param tokenParam
     * @return
     */
    @Override
    public String encryptToken(TokenParam tokenParam) {
        return TokenProvider.encryptToken(tokenParam);
    }

    /**
     * 解密令牌
     *
     * @param tokenEncrypted
     * @return
     */
    @Override
    public TokenParam decryptToken(String tokenEncrypted) {
        return TokenProvider.decryptToken(tokenEncrypted);
    }

    /**
     * 移除令牌
     *
     * @param adminId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean removeToken(Long adminId) {
        return agentAdminTokenDAO.deleteById(adminId) == 1;
    }

    /**
     * 修改密码
     *
     * @param adminId 管理员id
     * @param newPwd  新密码
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean changePwd(Long adminId, String newPwd) {
        return adminSecurityDAO.updatePassword(adminId, encryptPassword(newPwd)) == 1;
    }

    public static String encryptPassword(String password) {
        return SecureUtil.sha256(SecureUtil.md5(password).toUpperCase()).toLowerCase();
    }
}
