package com.starbuds.server.api.agent.service.admin.impl;

import com.starbuds.server.api.agent.common.pojo.admin.AgentAdminInfoVO;
import com.starbuds.server.api.agent.common.pojo.db.AgentAdminDO;
import com.starbuds.server.api.agent.common.pojo.db.AgentAdminSecurityDO;
import com.starbuds.server.api.agent.common.service.admin.AgAdminService;
import com.starbuds.server.api.agent.service.admin.dao.AgentAdminDAO;
import com.starbuds.server.api.agent.service.admin.dao.AgentAdminSecurityDAO;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 13:59
 * @Description
 */
@Slf4j
@Service
public class AgAdminServiceImpl implements AgAdminService {

    @Autowired
    AgentAdminDAO agentAdminDAO;

    @Autowired
    AgentAdminSecurityDAO agentAdminSecurityDAO;

    @Override
    public AgentAdminInfoVO getInfo(Long adminId) {
        return agentAdminDAO.selectVOById(adminId);
    }

    @Override
    public boolean isAdminExist(Long adminId) {
        return agentAdminDAO.countById(adminId) == 1;
    }

    /**
     * 创建管理员
     *
     * @param infoVO 管理员
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createAdmin(AgentAdminInfoVO infoVO) {
        AgentAdminDO adminDO = new AgentAdminDO();
        BeanUtils.copyProperties(infoVO, adminDO);
        if (agentAdminDAO.insert(adminDO) != 1) {
            throw new RuntimeException("创建管理员失败");
        }
        AgentAdminSecurityDO securityDO = new AgentAdminSecurityDO();
        securityDO.setAdminId(adminDO.getAdminId());
        securityDO.setAdminAccount(infoVO.getAdminPhone());
        securityDO.setAdminIsLocked(0);
        securityDO.setRoleId(0L);
        securityDO.setAdminPass(infoVO.getAdminPass());
        securityDO.setStatus(CommonStatus.Normal);
        if (agentAdminSecurityDAO.insert(securityDO) != 1) {
            throw new RuntimeException("保存密码失败");
        }
        return true;
    }

    /**
     * 编辑管理员
     *
     * @param infoVO 管理员
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean editAdmin(AgentAdminInfoVO infoVO) {
        AgentAdminDO adminDO = new AgentAdminDO();
        BeanUtils.copyProperties(infoVO, adminDO);
        return agentAdminDAO.updateById(adminDO) == 1;
    }

    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean disableAdmin(Long adminId) {
        return agentAdminDAO.updateStatus(adminId, CommonStatus.Disabled) == 1;
    }

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean enableAdmin(Long adminId) {
        return agentAdminDAO.updateStatus(adminId, CommonStatus.Normal) == 1;
    }

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteAdmin(Long adminId) {
        return agentAdminDAO.updateStatus(adminId, CommonStatus.Deleted) == 1;
    }

    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageNum    当前页码
     * @param pageSize   分页大小
     * @param agentId    公会id
     * @return result对象
     */
    @Override
    public PageList<AgentAdminInfoVO> getAdminPageList(String adminName, String adminPhone, Integer status, int pageNum, int pageSize, Long agentId) {
        PageList<AgentAdminInfoVO> page = new PageList<>(pageNum, pageSize);
        page.setList(agentAdminDAO.getAdminPageList(adminName, adminPhone, status, page.getPageCursor(), page.getPageSize(), agentId));
        page.setTotal(agentAdminDAO.countAdminPageList(adminName, adminPhone, status, agentId));
        return page;
    }


    /**
     * 判断是否存在账号
     *
     * @param adminPhone 手机号
     * @return 是否存在
     */
    @Override
    public boolean isPhoneExist(String adminPhone) {
        return agentAdminSecurityDAO.countByPhone(adminPhone) > 0;
    }

    /**
     * 是否可用
     *
     * @param adminId
     * @return
     */
    @Override
    public boolean isAdminEnable(Long adminId) {
        return Optional.ofNullable(agentAdminDAO.getAdminStatus(adminId)).orElse(-1) == 1;
    }

    @Override
    public Long getAgentId(Long adminId) {
        return agentAdminDAO.selectAgentId(adminId);
    }
}
