package com.starbuds.server.api.agent.v1.impl;

import cn.hutool.core.util.RandomUtil;
import com.starbuds.server.api.agent.common.pojo.admin.AgentAdminInfoVO;
import com.starbuds.server.api.agent.common.pojo.admin.TokenParam;
import com.starbuds.server.api.agent.common.pojo.db.AgentAdminSecurityDO;
import com.starbuds.server.api.agent.common.service.admin.AgAdminSecurityService;
import com.starbuds.server.api.agent.common.service.admin.AgAdminService;
import com.starbuds.server.api.agent.inf.PermitGuest;
import com.starbuds.server.api.agent.service.admin.impl.AgAdminSecurityServiceImpl;
import com.starbuds.server.api.agent.v1.facade.AdminFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;

/**
 * 管理员管理
 *
 * @author jiangxiaowei
 * @date 2020/1/16 14:05
 */
@Path("/v1/admin")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class AdminFacadeImpl extends BaseFacade implements AdminFacade {

    @Autowired
    AgAdminSecurityService agAdminSecurityService;

    @Autowired
    AgAdminService agAdminService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    /**
     * 账号密码登录
     *
     * @param account
     * @param password
     * @return
     */
    @POST
    @Path("login")
    @PermitGuest
    @Override
    public Result login(@FormParam("account") String account,
                        @FormParam("password") String password) {
        AgentAdminSecurityDO adminSecurityDO = agAdminSecurityService.getSecurityByAccount(account);
        if (adminSecurityDO == null || !agAdminSecurityService.checkPass(password, adminSecurityDO.getAdminPass())) {
            return new Result(GlobalApiError.OperationFailed, "账号或密码错误");
        }

        if (!agAdminService.isAdminEnable(adminSecurityDO.getAdminId())) {
            return new Result(GlobalApiError.OperationFailed, "该账号已被停用");
        }

        // 创建令牌登录
        String token = RandomUtil.randomString(64);
        if (!agAdminSecurityService.setToken(adminSecurityDO.getAdminId(), this.getHeader().getClientType().getValue(), token)) {
            return new Result(GlobalApiError.OperationFailed, "令牌创建失败");
        }
        // 加密令牌
        TokenParam tokenParam = new TokenParam();
        tokenParam.setAdminId(adminSecurityDO.getAdminId());
        tokenParam.setRoleId(adminSecurityDO.getRoleId());
        tokenParam.setToken(token);
        tokenParam.setTimestamp(System.currentTimeMillis());
        tokenParam.setAgentId(agAdminService.getAgentId(adminSecurityDO.getAdminId()));
        String tokenEncrypted = agAdminSecurityService.encryptToken(tokenParam);

        MutableResult mResult = new MutableResult();
        mResult.put("token", tokenEncrypted);
        return mResult.build();
    }

    /**
     * 个人信息
     *
     * @return
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo() {
        return new Result(agAdminService.getInfo(this.getToken().getAdminId()));
    }

    /**
     * 重置密码
     *
     * @param adminId 被重置人的id
     * @param newPwd  新密码
     * @return result对象
     */
    @POST
    @Path("resetPwd")
    @Override
    public Result resetPwd(@FormParam("adminId") Long adminId,
                           @FormParam("newPwd") String newPwd) {
        if (!agAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }

        if (agAdminSecurityService.changePwd(adminId, newPwd)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }


    /**
     * 修改密码
     *
     * @param oldPwd 旧密码
     * @param newPwd 新密码
     * @return result对象
     */
    @POST
    @Path("changePwd")
    @Override
    public Result changePwd(@FormParam("oldPwd") String oldPwd,
                            @FormParam("newPwd") String newPwd) {
        AgentAdminSecurityDO dbSecurity = agAdminSecurityService.getSecurityByAdminId(this.getToken().getAdminId());
        //校验旧密码
        if (dbSecurity == null || !agAdminSecurityService.checkPass(oldPwd, dbSecurity.getAdminPass())) {
            return new Result(GlobalApiError.OperationFailed, "旧密码不正确");
        }
        if (agAdminSecurityService.changePwd(this.getToken().getAdminId(), newPwd)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 创建管理员
     *
     * @param adminName   管理员名称
     * @param adminPhone  手机号
     * @param adminAvatar 头像
     * @param adminEmail  email
     * @param adminPass   密码
     * @param status      状态
     * @return result对象
     */
    @POST
    @Path("createAdmin")
    @Override
    public Result createAdmin(@FormParam("adminName") String adminName,
                              @FormParam("adminPhone") String adminPhone,
                              @FormParam("adminAvatar") String adminAvatar,
                              @FormParam("adminEmail") String adminEmail,
                              @FormParam("adminPass") String adminPass,
                              @FormParam("status") Integer status,
                              @FormParam("agentId") Long agentId) {
        if (agAdminService.isPhoneExist(adminPhone)) {
            return new Result(GlobalApiError.OperationFailed, "该手机号已是管理员");
        }
        AgentAdminInfoVO infoVO = new AgentAdminInfoVO();
        infoVO.setAdminId(identityService.getId());
        infoVO.setAdminName(adminName);
        infoVO.setAdminPhone(adminPhone);
        infoVO.setAdminAvatar(adminAvatar);
        infoVO.setAdminCreateTime(System.currentTimeMillis());
        infoVO.setAdminEmail(adminEmail);
        infoVO.setAdminStatus(status);
        infoVO.setAdminPass(AgAdminSecurityServiceImpl.encryptPassword(adminPass));
        infoVO.setAgentId(agentId);
        if (agAdminService.createAdmin(infoVO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageNum    当前页码
     * @param pageSize   分页大小
     * @param agentId    公会id
     * @return result对象
     */
    @GET
    @Path("getAdminPageList")
    @Override
    public Result getAdminPageList(@QueryParam("adminName") String adminName,
                                   @QueryParam("adminPhone") String adminPhone,
                                   @QueryParam("status") Integer status,
                                   @QueryParam("pageNum") int pageNum,
                                   @QueryParam("pageSize") int pageSize,
                                   @QueryParam("agentId") Long agentId) {
        return new Result(agAdminService.getAdminPageList(adminName, adminPhone, status, pageNum, pageSize, agentId));
    }

    /**
     * 编辑管理员
     *
     * @param adminId     管理员id
     * @param adminName   管理员名称
     * @param adminAvatar 头像
     * @param adminEmail  email
     * @param status      状态
     * @return result对象
     */
    @POST
    @Path("editAdmin")
    @Override
    public Result editAdmin(@FormParam("adminId") Long adminId,
                            @FormParam("adminName") String adminName,
                            @FormParam("adminAvatar") String adminAvatar,
                            @FormParam("adminEmail") String adminEmail,
                            @FormParam("status") Integer status) {
        if (!agAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        AgentAdminInfoVO infoVO = new AgentAdminInfoVO();
        infoVO.setAdminId(adminId);
        infoVO.setAdminName(adminName);
        infoVO.setAdminAvatar(adminAvatar);
        infoVO.setAdminEmail(adminEmail);
        infoVO.setAdminStatus(status);
        if (agAdminService.editAdmin(infoVO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    @POST
    @Path("disableAdmin")
    @Override
    public Result disableAdmin(@FormParam("adminId") Long adminId) {
        if (!agAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        if (agAdminService.disableAdmin(adminId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    @POST
    @Path("enableAdmin")
    @Override
    public Result enableAdmin(@FormParam("adminId") Long adminId) {
        if (!agAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        if (agAdminService.enableAdmin(adminId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    @POST
    @Path("deleteAdmin")
    @Override
    public Result deleteAdmin(@FormParam("adminId") Long adminId) {
        if (!agAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        if (agAdminService.deleteAdmin(adminId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

}
