package com.starbuds.server.api.agent.v1.impl;

import com.starbuds.server.api.agent.common.service.admin.AgAgentService;
import com.starbuds.server.api.agent.common.service.admin.AgAnchorService;
import com.starbuds.server.api.agent.v1.facade.AnchorFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 公会业务实现
 *
 * @author jiangxiaowei
 * @date 2020/1/15 19:07
 */
@Path("/v1/anchor")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class AnchorFacadeImpl extends BaseFacade implements AnchorFacade {

    @Autowired
    AgAgentService agAgentService;

    @Resource
    AgAnchorService agAnchorService;

    /**
     * 通过
     *
     * @param auditId
     * @param auditMsg
     * @param anchorRatio
     * @return
     */
    @POST
    @Path("passAudit")
    @Override
    public Result passAudit(@FormParam("auditId") Long auditId,
                            @FormParam("auditMsg") String auditMsg,
                            @FormParam("anchorRatio") Double anchorRatio) {
        if (!agAgentService.isAuditExist(auditId)) {
            return new Result(GlobalApiError.OperationFailed, "申请记录不存在");
        }

        if (agAgentService.getAuditStatus(auditId) != AuditStatusEnum.WAIT.getCode()) {
            return new Result(GlobalApiError.OperationFailed, "该申请已被处理，请勿重复操作");
        }

        if (agAgentService.passAudit(auditId, this.getToken().getAdminId(), auditMsg, anchorRatio)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 拒绝
     *
     * @param auditId
     * @param auditMsg
     * @return
     */
    @POST
    @Path("denyAudit")
    @Override
    public Result denyAudit(@FormParam("auditId") Long auditId,
                            @FormParam("auditMsg") String auditMsg) {
        if (!agAgentService.isAuditExist(auditId)) {
            return new Result(GlobalApiError.OperationFailed, "申请记录不存在");
        }
        if (agAgentService.denyAudit(auditId, this.getToken().getAdminId(), auditMsg)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取主播列表
     *
     * @param userName     用户姓名
     * @param startDate    开始时间
     * @param endDate      结束时间
     * @param liveStatus   直播状态
     * @param streamStatus 推流状态
     * @param userNo       星芽号
     * @param userPhone    手机号
     * @param charmLevel   主播等级
     * @param pageNum      当前页码
     * @param pageSize     分页大小
     * @param orderField   排序字段
     * @param isAsc        是否升序
     * @return result对象
     */
    @GET
    @Path("getAnchorPageList")
    @Override
    public Result getAnchorPageList(@QueryParam("userName") String userName,
                                    @QueryParam("startDate") Long startDate,
                                    @QueryParam("endDate") Long endDate,
                                    @QueryParam("liveStatus") Integer liveStatus,
                                    @QueryParam("streamStatus") Integer streamStatus,
                                    @QueryParam("userNo") Long userNo,
                                    @QueryParam("userPhone") String userPhone,
                                    @QueryParam("charmLevel") Integer charmLevel,
                                    @QueryParam("pageNum") int pageNum,
                                    @QueryParam("pageSize") int pageSize,
                                    @QueryParam("orderField") String orderField,
                                    @QueryParam("isAsc") boolean isAsc) {
        if (StringUtils.isEmpty(orderField) || containsSqlInjection(orderField)) {
            orderField = "userId";
        }
        return new Result(agAnchorService.getAnchorPageList(userName, this.getToken().getAgentId(), startDate, endDate, liveStatus, streamStatus, userNo, userPhone, charmLevel, pageNum, pageSize, orderField, isAsc));
    }

    /**
     * 获取入会申请列表
     *
     * @param agentId
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GET
    @Path("getAuditPageList")
    @Override
    public Result getAuditPageList(@QueryParam("agentId") Long agentId,
                                   @QueryParam("applyType") Integer applyType,
                                   @QueryParam("userId") Long userId,
                                   @QueryParam("auditStatus") Integer auditStatus,
                                   @QueryParam("userName") String userName,
                                   @QueryParam("pageNum") int pageNum,
                                   @QueryParam("pageSize") int pageSize) {
        return new Result(agAgentService.getAuditPageList(agentId, applyType, userId, auditStatus, userName, pageNum, pageSize));
    }

    /**
     * 获取公会详情
     *
     * @param agentId 公会id
     * @return 公会详情
     */
    @GET
    @Path("getAgentDetail")
    @Override
    public Result getAgentDetail(@QueryParam("agentId") Long agentId) {
        if (!agAgentService.isAgentExist(agentId)) {
            return new Result(GlobalApiError.OperationFailed, "公会不存在");
        }
        return new Result(agAgentService.getAgentDetail(agentId));
    }

    /**
     * 获取公会主播分成比例
     *
     * @param agentId
     * @return
     */
    @GET
    @Path("getAnchorRatio")
    @Override
    public Result getAnchorRatio(@QueryParam("agentId") Long agentId) {
        if (!agAgentService.isAgentExist(agentId)) {
            return new Result(GlobalApiError.OperationFailed, "公会不存在");
        }
        return new Result(agAgentService.getAnchorRatio(agentId));
    }

    /**
     * 设置主播分成比例
     *
     * @param signId
     * @param ratio
     * @return
     */
    @POST
    @Path("setAnchorRatio")
    @Override
    public Result setAnchorRatio(@FormParam("signId") Long signId, @FormParam("ratio") Double ratio) {
        if (!agAgentService.isSignExist(signId)) {
            return new Result(GlobalApiError.OperationFailed, "签订id不存在");
        }
        if (agAgentService.setAnchorRatio(signId, ratio)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 直播记录列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getLiveHisPageList")
    @Override
    public Result getLiveHisPageList(@QueryParam("userId") Long userId, @QueryParam("pageIndex") int pageIndex, @QueryParam("pageSize") int pageSize) {

        if (!this.getToken().getAgentId().equals(agAnchorService.getAgentId(userId))) {
            return new Result(GlobalApiError.OperationFailed, "只可查看本公会下主播");
        }

        return new Result(agAnchorService.getLiveHisPageList(userId, pageIndex, pageSize));
    }

    /**
     * 获取主播详情
     *
     * @param userId 用户id
     * @return result对象
     */
    @GET
    @Path("getAnchorDetail")
    @Override
    public Result getAnchorDetail(@QueryParam("userId") Long userId) {

        if (!this.getToken().getAgentId().equals(agAnchorService.getAgentId(userId))) {
            return new Result(GlobalApiError.OperationFailed, "只可查看本公会下主播");
        }

        return new Result(agAnchorService.getAnchorDetail(userId));
    }

    /**
     * 获取主播认证记录
     *
     * @param userId 用户id
     * @return result对象
     */
    @GET
    @Path("getAnchorAuditRecord")
    @Override
    public Result getAnchorAuditRecord(@QueryParam("userId") Long userId) {

        if (!this.getToken().getAgentId().equals(agAnchorService.getAgentId(userId))) {
            return new Result(GlobalApiError.OperationFailed, "只可查看本公会下主播");
        }

        return new Result(agAnchorService.getAnchorAuditRecord(userId));
    }

    /**
     * 获取推流记录
     *
     * @param liveHisId
     * @return
     */
    @GET
    @Path("getLiveStreamHis")
    @Override
    public Result getLiveStreamHis(@QueryParam("liveHisId") Long liveHisId) {
        return new Result(agAnchorService.getLiveStreamHis(liveHisId));
    }

    /**
     * 是否含有sql注入，返回true表示含有
     *
     * @param sql
     * @return 是否包含sql注入
     */
    public static boolean containsSqlInjection(String sql) {
        Pattern pattern = Pattern.compile("\\b(and|exec|insert|select|drop|grant|alter|delete|update|count|chr|mid|master|truncate|char|declare|or)\\b|(\\*|;|\\+|'|%)");
        Matcher matcher = pattern.matcher(sql);
        return matcher.find();
    }
}
